/*jslint nomen: true, maxlen: 200, indent: 2*/
/*global rJS, console, window, document, RSVP, jIO*/

(function (window, rJS, jIO) {
  "use strict";

  function downloadJSON(url) {
    return RSVP.Queue()
      .push(function () {
        return jIO.util.ajax({
          url: url,
          dataType: "json"
        });
      })
      .push(undefined, function (error) {
        console.log(error);
        throw error;
      })
      .push(function (evt) {
        return evt.target.response;
      });
  }

  function render_form(gadget, scope, schema) {
    return gadget.getDeclaredGadget(scope)
      .push(function (g) {
        return g.render({
          schema: schema
        });
      });
  }

  function render_schema_selection_form(gadget, url) {
    if (gadget.props.schemas.indexOf(url) < 0) {
      gadget.props.schemas.push(url);
    }
    return render_form(gadget, "schema", {
      default: url,
      enum: gadget.props.schemas
    });
  }

  function render_document_selection_form(gadget) {
    return render_form(gadget, "document", {
      default: gadget.props.documents[0],
      enum: gadget.props.documents
    });
  }

  rJS(window)
    .ready(function (g) {
      g.props = {};
      return RSVP.Queue()
        .push(function () {
          return RSVP.all([
            downloadJSON("schemas.json"),
            downloadJSON("docs.json")
          ]);
        })
        .push(function (arr) {
          var schemas = arr[0],
            docs = arr[1];
          g.props.schemas = docs.filter(function (v) {
            var suffix = "_schema.json";
            return v.indexOf(suffix, v.length - suffix.length) !== -1;
          });
          g.props.schemas = g.props.schemas.concat([
            "http://json-schema.org/draft-07/schema",
            "http://json-schema.org/draft-06/schema",
            "http://json-schema.org/draft-04/schema"
          ]);
          g.props.schemas = g.props.schemas.concat(schemas);
          g.props.schemas = ["form generated by schema"].concat(g.props.schemas);
          g.props.documents = docs.concat(schemas);
          g.props.documents = ["open document"].concat(g.props.documents);
        })
        .push(function () {
          return RSVP.all([
            render_form(g, "schema", {
              type: "string",
              default: g.props.schemas[0],
              enum: g.props.schemas
            }),
            render_document_selection_form(g)
          ]);
        });
    })
    .allowPublicAcquisition("notifyValid", function () {
      return;
    })
    .allowPublicAcquisition("notifyInvalid", function () {
      return;
    })
    .allowPublicAcquisition("notifyChange", function (arr, scope) {
      var gadget = this;
      if (scope === "schema") {
        return this.getDeclaredGadget("schema")
          .push(function (g) {
            return g.getContent();
          })
          .push(function (ret) {
            if (ret !== gadget.props.schemas[0]) {
              return gadget.changeState({
                schema_url: ret,
                json_document: JSON.parse(
                  document.getElementById("json_document_content").textContent
                )
              });
            }
          });
      }
      if (scope === "document") {
        return gadget.getDeclaredGadget("document")
          .push(function (g) {
            return g.getContent();
          })
          .push(function (ret) {
            if (ret !== gadget.props.documents[0]) {
              return downloadJSON(ret);
            }
            return gadget.state.json_document;
          })
          .push(function (json_document) {
            var schema_url = gadget.state.schema_url,
              tasks = [];
            if (json_document.$schema) {
              if (json_document.$schema !== schema_url) {
                tasks.push(render_schema_selection_form(gadget, json_document.$schema));
              }
              schema_url = json_document.$schema;
            }
            tasks.push(gadget.changeState({
              schema_url: schema_url,
              json_document: json_document
            }));
            return RSVP.all(tasks);
          })
          .push(function () {
            return render_document_selection_form(gadget);
          });
      }
      if (scope === "form_view") {
        return this.getDeclaredGadget("form_view")
          .push(function (g) {
            return g.getContent();
          })
          .push(function (ret) {
            if (ret === undefined) {
              ret = {};
            }
            document.getElementById("json_document_content").textContent =
              JSON.stringify(ret, null, "  ");
          });
      }
      return;
    })
    .onStateChange(function () {
      var g = this;
      return g.getDeclaredGadget("form_view")
        .push(function (form_view) {
          return form_view.render({
            value: g.state.json_document,
            schema_url: g.state.schema_url
          })
            .push(function () {
              return form_view.getContent();
            })
            .push(function (ret) {
              if (ret === undefined) {
                ret = {};
              }
              g.state.json_document = ret;
              document.getElementById("json_document_content").textContent =
                JSON.stringify(ret, null, "  ");
            });
        });
    });

}(window, rJS, jIO));